/***************************************************************************
*			FeiTian Epass Editor
****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "epassapi.h"
#include "md5.h"

EPAS_HANDLE handle;
unsigned long SerialNumber[2];

unsigned char buff[512];

unsigned long
ErrorInfo( char *        msg,
           unsigned long status )
{
    printf( "\n%s: ", msg );
    switch( status )
    {
        case FT_SUCCESS             : printf("SUCCESS"); break;
        case FT_CANNOT_OPEN_DRIVER  : printf("CANNOT_OPEN_DRIVER"); break;
        case FT_INVALID_DRVR_VERSION: printf("INVALID_DRVR_VERSION"); break;
        case FT_INVALID_COMMAND     : printf("INVALID_COMMAND"); break;
        case FT_ACCESS_DENIED       : printf("ACCESS_DENIED"); break;
        case FT_ALREADY_ZERO        : printf("ALREADY_ZERO"); break;
        case FT_UNIT_NOT_FOUND      : printf("UNIT_NOT_FOUND"); break;
        case FT_DEVICE_REMOVED      : printf("DEVICE_REMOVED"); break;
        case FT_COMMUNICATIONS_ERROR: printf("COMMUNICATIONS_ERROR"); break;
        case FT_DIR_NOT_FOUND       : printf("DIR_NOT_FOUND"); break;
        case FT_FILE_NOT_FOUND      : printf("FILE_NOT_FOUND"); break;
        case FT_MEM_CORRUPT         : printf("MEM_CORRUPT"); break;
        case FT_INTERNAL_HW_ERROR   : printf("INTERNAL_HW_ERROR"); break;
        case FT_INVALID_RESP_SIZE   : printf("INVALID_RESP_SIZE"); break;
        case FT_PIN_EXPIRED         : printf("PIN_EXPIRED"); break;
        case FT_ALREADY_EXISTS      : printf("ALREADY_EXISTS"); break;
        case FT_NOT_ENOUGH_MEMORY   : printf("NOT_ENOUGH_MEMORY"); break;
        case FT_INVALID_PARAMETER   : printf("INVALID_PARAMETER"); break;
        case FT_INPUT_TOO_LONG      : printf("INPUT_TOO_LONG"); break;
//        case FT_ALIGNMENT_ERROR     : printf("ALIGNMENT_ERROR"); break;
        case FT_INVALID_STATUS      : printf("INVALID_STATUS"); break;
        case FT_INVALID_FILE_SELECTED : printf("INVALID_FILE_SELECTED"); break;
        case FT_DEVICE_IN_USE       : printf("DEVICE_IN_USE"); break;
        case FT_INVALID_API_VERSION : printf("INVALID_API_VERSION"); break;
        case FT_TIME_OUT_ERROR      : printf("TIME_OUT_ERROR"); break;
        case FT_ITEM_NOT_FOUND      : printf("ITEM_NOT_FOUND"); break;
        case FT_COMMAND_ABORTED     : printf("COMMAND_ABORTED"); break;
        default: printf( "UNKNOWN ERROR[%04lX]", status ); break;
    }

    return status;
}

void PutInt32(
	unsigned char * Buffer,
	unsigned long   value )
{
	Buffer[0] = (unsigned char)(value);
	Buffer[1] = (unsigned char)(value >> 8);
	Buffer[2] = (unsigned char)(value >> 16);
	Buffer[3] = (unsigned char)(value >> 24);
}

void
HexDisplay( char *          msg,
            unsigned char * data,
            unsigned long   length )
{
    unsigned long i, n;

    printf( "\n%s", msg );

    while( length > 0 )
    {
        n = (length > 8) ? 8 : length;
        for( i = 0; i < n; i++ )
            printf( "%02X ", data[i] );

        while( i++ < 8 ) printf( "   " );

        printf( "  [" );
        for( i = 0; i < n; i++, data++ )
            printf( "%c", (*data < 32 || *data > 127) ? '.' : *data );

        while( i++ < 8 ) printf( " " );
        printf( "]" );

        length -= n;
        if( length > 0 )
        {
            printf( "\n" );
            for( n = strlen(msg); n > 0; n-- ) printf( " " );
        }
    }
}

char *
getString( char *buf, int len, char *prompt )
{
    fprintf( stderr, "%s", prompt );
    fflush( stdin );
    fgets( buf, len, stdin );

    // remove line feed at the end (if any)
    if( buf[strlen(buf)-1] == '\n' ) buf[strlen(buf)-1] = '\0';
    return buf;
}

unsigned long
getHexNum( char *prompt )
{
    unsigned long n = 0;
    char buf[10];
    sscanf( getString( buf, sizeof(buf), prompt ), "%lx", &n );
    return n;
}

unsigned long
getNumber( char *prompt )
{
    unsigned long n = 0;
    char buf[10];
    sscanf( getString( buf, sizeof(buf), prompt ), "%ld", &n );
    return n;
}

int
HexByte( char c )
{
    if( c >= '0' && c <= '9' ) return c - '0';
    if( c >= 'A' && c <= 'F' ) return c - 'A' + 10;
    if( c >= 'a' && c <= 'f' ) return c - 'a' + 10;
    return -1;
}

unsigned char *
GetHexString( unsigned char *Data, int len, char *prompt )
{
    int i, x;
    char *p;
    char buff[256];

    getString( buff, 256, prompt );
    for( i = 0, p = buff; i < len; )
    {
        if( *p == ' ' || *p == '\t' || *p == '-' )
        {
            p++;
            continue;
        }
        x = (HexByte( p[0] ) << 4) | HexByte( p[1] );
        if( x < 0 )
            return NULL;

        Data[i++] = (unsigned char)x;
        p += 2;
    }
    return Data;
}

int
GetMenuItem( char *prompt )
{
    int c;

    printf("\n");
    printf("\n%s\n\nSelection ....................? ", prompt );
    fflush(stdin );
    fflush(stderr);
    fflush(stdout);

    c=fgetc(stdin);
    getchar();
    return( toupper((int)c));
}

char *
GetFileTypeText( unsigned char Type )
{
    switch( Type )
    {
        case EPAS_FILETYPE_UNUSED  : return "FREE";
        case EPAS_FILETYPE_DIR     : return "DIR ";
        case EPAS_FILETYPE_DATA    : return "DATA";
        case EPAS_FILETYPE_COUNTER : return "CNTR";
        case EPAS_FILETYPE_KEY     : return "KEY ";
    }

    return "????";
}

char *
GetFileAccessText( unsigned char access )
{
    switch( access )
    {
        case EPAS_ACCESS_ANYONE    : return "ALWAYS";
        case EPAS_ACCESS_USER      : return "PIN   ";
        case EPAS_ACCESS_OFFICER   : return "SO-PIN";
        case EPAS_ACCESS_NONE      : return "NEVER ";
    }
    return "????  ";
}

char *
YesNo( unsigned long value )
{
    return value ? "Yes" : "No ";
}

unsigned long
epas_EnumDirCallBack( EPAS_HANDLE    ContextHandle,
                      unsigned long  DirId,
                      unsigned long  index,
                      PEPAS_FILEINFO FileInfo )
{
    printf( "\n%08lX %08lX %s %s %s %s %-6ld",
            DirId,
            FileInfo->ulID,
            GetFileTypeText(FileInfo->ucFileType),
            GetFileAccessText(FileInfo->ucReadAccess),
            GetFileAccessText(FileInfo->ucWriteAccess),
            GetFileAccessText(FileInfo->ucCryptAccess),
            FileInfo->ulFileSize );

    if( FileInfo->ucFileType == EPAS_FILETYPE_DIR )
    {
        unsigned char AppGuid[16], AppName[EPAS_APP_NAME_SIZE];

        epas_ChangeDir( ContextHandle, EPAS_DIR_BY_LONG_ID, FileInfo->ulID, NULL );
        if( epas_GetProperty( ContextHandle,EPAS_PROP_APP_NAME,NULL,AppName,sizeof(AppName)) == FT_SUCCESS )
        {
            printf( " %s", AppName );
        }

        epas_ChangeDir( ContextHandle,
                        EPAS_DIR_BY_LONG_ID,
                        FileInfo->ulID,
                        NULL );
        if( epas_GetProperty( ContextHandle,
                              EPAS_PROP_APP_GUID,
                              NULL,
                              AppGuid,
                              sizeof(AppGuid) ) == FT_SUCCESS )
        {
            // 16 byte application GUID present
            unsigned long i;
            printf( "\n   GUID: " );
            for( i = 0; i < 16; i++ )
            {
                printf( "%02X", AppGuid[i] );
                if( i >= 3 && i <= 9 && i & 1 ) printf( "-" );
            }
        }

        // Restore current directory
        epas_ChangeDir( ContextHandle, EPAS_DIR_BY_LONG_ID, DirId, NULL );
    }
    return 0;
}

EPAS_STATUS epas_EnumDirEntries(
	EPAS_HANDLE   ContextHandle,
    unsigned long Dir_Id )
{
    unsigned long i;
    EPAS_FILEINFO fi;
    EPAS_STATUS status;

    status = epas_ChangeDir( ContextHandle,EPAS_DIR_BY_LONG_ID,Dir_Id,NULL );
    if( status != FT_SUCCESS ) return status;

    for( i = 0; i < 256; i++ )
    {
        status = epas_Directory( ContextHandle, 0, i, &fi, sizeof(fi) );
        if( status != FT_SUCCESS ) break;

        if( epas_EnumDirCallBack( ContextHandle, Dir_Id, i, &fi ) ) break;

        if( fi.ucFileType == EPAS_FILETYPE_DIR )
		{
            epas_EnumDirEntries( ContextHandle, fi.ulID );

			// Restore current directory
			status = epas_ChangeDir( ContextHandle,EPAS_DIR_BY_LONG_ID,Dir_Id,NULL );
			if( status != FT_SUCCESS ) return status;
		}

        if( fi.ucFileType == EPAS_FILETYPE_UNUSED ) break;
    }
	return FT_SUCCESS;
}

void CryptMenu( void )
{
    EPAS_FILEINFO fi;
    EPAS_STATUS status;
    unsigned long len, k1_id, k2_id;
    unsigned char text[80];
    unsigned char digest[16], digest2[16], key1[16], key2[16];
    unsigned char *p;
    int menuitem;

    menuitem = GetMenuItem(
        "\nHASH:  (H)MAC  (M)akeHMACKeys  "
        );

    switch( menuitem )
    {
        case 'M':
            printf( "\nThis command creates two set of tokens for HMAC-MD5 authentication.\n\n" );
            k1_id = getHexNum( "Token1 ID .............. (0..FFFF): " );
            k2_id = getHexNum( "Token2 ID .............. (0..FFFF): " );
            p     = (unsigned char*)getString((char*) buff, sizeof(buff),
                               "HMAC-MD5 key .............. (text): " );
            len   = strlen((char*)p);

            MD5_HMAC( NULL, 0, p, len, key1, key2, digest );

            HexDisplay( " MD5_HMAC TOKEN#1: ", key1, 16 );
            HexDisplay( "          TOKEN#2: ", key2, 16 );

            status = epas_OpenFile( handle,
                                    EPAS_FILE_WRITE,
                                    k1_id,
                                    &fi,
                                    sizeof(fi) );
            if( status == FT_FILE_NOT_FOUND )
            {
                memset( &fi, 0, sizeof(fi) );

                fi.ulID          = k1_id;
                fi.ulFlags       = 0;
                fi.ulFileSize    = 16;
                fi.ucFileType    = EPAS_FILETYPE_KEY;
                fi.ucWriteAccess = EPAS_ACCESS_OFFICER;
                fi.ucCryptAccess = EPAS_ACCESS_ANYONE;
                fi.ucReadAccess  = EPAS_ACCESS_NONE;

                status = epas_CreateFile( handle, 0, &fi, sizeof(fi) );
            }

            if( status == FT_SUCCESS )
                status = epas_Write( handle, 0, 0, key1, 16, &len );

            if( status == FT_SUCCESS )
            {
                status = epas_OpenFile( handle,
                                        EPAS_FILE_WRITE,
                                        k2_id,
                                        &fi,
                                        sizeof(fi) );
                if( status == FT_FILE_NOT_FOUND )
                {
					memset( &fi, 0, sizeof(fi) );

                    fi.ulID          = k2_id;
                    fi.ulFlags       = 0;
                    fi.ulFileSize    = 16;
                    fi.ucFileType    = EPAS_FILETYPE_KEY;
                    fi.ucWriteAccess = EPAS_ACCESS_OFFICER;
                    fi.ucCryptAccess = EPAS_ACCESS_ANYONE;
                    fi.ucReadAccess  = EPAS_ACCESS_NONE;

                    status = epas_CreateFile( handle, 0, &fi, sizeof(fi) );
                }

                if( status == FT_SUCCESS )
                    status = epas_Write( handle, 0, 0, key2, 16, &len );

                if( status == FT_SUCCESS )
                    status = epas_CloseFile( handle );
            }
            ErrorInfo( "MakeKeys", status );

            break;

        case 'H':
            k1_id = getHexNum( "Key1 ID ................ (0..FFFF): " );
            k2_id = getHexNum( "Key2 ID ................ (0..FFFF): " );
                    getString( (char*)text, sizeof(text),
                               "Enter RCV ......... (max 55 chars): " );
            len = strlen((char*)text);

            if( len > 55 ) len = 55;
            status = epas_MD5_HMAC( handle,
                                    k1_id,
                                    k2_id,
                                    text,
                                    len,
                                    digest );    // Digest
            ErrorInfo( "HMAC-MD5", status );
            if( status != FT_SUCCESS ) break;

            HexDisplay( "HMAC-MD5(HW): ", digest, 16 );

            // Verify the response

            printf( "\n\nTo verify, enter original HMAC-MD5 key.\n\n" );
            p     = (unsigned char*)getString((char*) buff, sizeof(buff),
                               "HMAC-MD5 key .............. (text): " );

            MD5_HMAC( text, len, p, strlen((char*)p), NULL, NULL, digest2 );

            HexDisplay( "HMAC-MD5(SW): ", digest2, 16 );

            if( memcmp( digest, digest2, 16 ) != 0 )
            {
                printf( "\nVerification FAILED." );
            }
            else
            {
                printf( "\nVerification SUCCESSFUL." );
            }
            break;


        default : printf( "\nInvalid Command." );
    }
}

void
SetupMenu( void )
{
    EPAS_STATUS status;
    unsigned char Buffer[256];

    switch( GetMenuItem(
        "SETUP:  (P)in     (S)oPin   (T)okenName       (C)leanup\n"
		"        (N)ewPin            (A)ccessSettings  (D)eleteAll" ))
    {
        case 'C':
            status = epas_Cleanup( handle, 0 );
            ErrorInfo( "Cleanup", status );
            break;

        case 'P':
            PutInt32( Buffer+0, getHexNum( "Enter Old PIN: " ));
	    PutInt32( Buffer+4, getHexNum( "Enter New PIN: " ));
            status = epas_ChangeCode( handle,
                                      EPAS_CHANGE_USER_PIN,
                                      Buffer+0, 4,
                                      Buffer+4, 4 );
            ErrorInfo( "ModifyPIN", status );
            break;

        case 'N':
            getString( (char*)buff, sizeof(buff), "\nEnter SO PIN: " );

            PutInt32( Buffer, getHexNum( "New User PIN: " ));
            status = epas_ChangeCode( handle,
                                      EPAS_UNBLOCK_USER_PIN,
                                      buff,
                                      strlen((char*)buff),
                                      Buffer, 4 );
            ErrorInfo( "UnblockPIN", status );
            break;

        case 'A':
        {
            EPAS_ACCESSINFO accessInfo;

            // This function needs SOPIN verification first.
            accessInfo.ucMaxPinRetries = (unsigned char)
                getNumber( "Maximum PIN Retry Count ........................ (0..15): " );
            accessInfo.ucCurPinCounter = (unsigned char)
                getNumber( "Initial PIN Retry Count ........................ (0..15): " );
            accessInfo.ucCreateAccess  = (unsigned char)
                getNumber( "Create Access: ........ (0=Always 1=PIN 2=SOPIN 7=Never): " );
            accessInfo.ucDeleteAccess  = (unsigned char)
                getNumber( "Delete Access: ........ (0=Always 1=PIN 2=SOPIN 7=Never): " );

            status = epas_SetProperty( handle,
                                       EPAS_PROP_ACCESSINFO,
									   NULL,
                                       &accessInfo,
									   sizeof(accessInfo) );
            ErrorInfo( "SetAccessSettings", status );
            break;
        }

        case 'S':
            getString( (char*)buff, sizeof(buff), "\nEnter Cur PIN: " );
            getString( (char*)Buffer, sizeof(Buffer), "\nEnter New PIN: " );
            status = epas_ChangeCode( handle,
                                      EPAS_CHANGE_SO_PIN,
                                      buff, strlen((char*)buff),
                                      Buffer, strlen((char*)Buffer) );
            ErrorInfo( "Modify SO-PIN", status );
            break;

        case 'D':
            // This function needs SOPIN verification first.
            status = epas_DeleteDir( handle, EPAS_DIR_BY_LONG_ID, EPAS_ROOT_DIR, NULL );

            ErrorInfo( "DeleteAll", status );
            break;

        case 'T':
            getString( (char*)Buffer,
                       EPAS_FRIENDLY_NAME_SIZE,
                       "Device friendly name: " );
            status = epas_SetProperty( handle,
									   EPAS_PROP_FRIENDLY_NAME,
									   NULL,
									   Buffer,
									   strlen((char*)Buffer) );
            ErrorInfo( "SetDeviceName", status );
            break;
    }
}

void
FileMenu( void )
{
    EPAS_FILEINFO fi;
    EPAS_STATUS status;
    unsigned long i, len, df_id, ef_id, offs, cur_dir[2];
    unsigned char Buffer[256];
    unsigned char * p;

    if( epas_GetCurrentDir( handle, 0, cur_dir, 2, &len ) == FT_SUCCESS )
    {
        if( len > 1 )
            printf( "\nCurrent Dir: %lx/%lx", cur_dir[0], cur_dir[1] );
        else
            printf( "\nCurrent Dir: %lx", cur_dir[0] );
    }

    for( ;; )
    switch( GetMenuItem(
        "FILE:  C(h)Dir  (R)ead   Create(F)ile D(e)lFile     Create(A)pp Cr(y)ptMenu\n"
        "       (O)pen   (W)rite  (L)ist       Create(D)ir   DelD(i)r    Set(u)pMenu\n"
        "       (C)lose  Write(B)in            E(x)it" ))
    {
        case 'H':
            df_id = getHexNum( "Dir ID ............. (0..FFFF): " );
			status = epas_ChangeDir( handle, EPAS_DIR_BY_LONG_ID, df_id, NULL );
            ErrorInfo( "ChangeDir", status );
			break;

        case 'O':
            ef_id = getHexNum( "File ID ............ (0..FFFF): " );
            status = epas_OpenFile(
                handle,
                0,
                ef_id,
                &fi,
                sizeof(fi) );
            ErrorInfo( "OpenFile", status );
            if( status == FT_SUCCESS )
            {
                printf( "\nFile Size ....................: %ld",
                        fi.ulFileSize );
                printf( "\nFile Type ....................: %s",
                        GetFileTypeText( fi.ucFileType ));
                printf( "\nRead Access ..................: %s",
                        YesNo( fi.ucGrantedAccess & EPAS_ACCESS_READ ));
                printf( "\nWrite Access .................: %s",
                        YesNo( fi.ucGrantedAccess & EPAS_ACCESS_WRITE ));
                printf( "\nCrypt Access .................: %s",
                        YesNo( fi.ucGrantedAccess & EPAS_ACCESS_CRYPT ));
            }
            break;

        case 'C':
            status = epas_CloseFile( handle );
            ErrorInfo( "ClosetFile", status );
            break;

        case 'R' :
            offs = getNumber( "File Offset ......... (0..65534): " );
            len  = getNumber( "Number of bytes ....... (2..256): " );
            if( len > sizeof(Buffer) )
            {
                printf( "\nMaximum of 256 (100h) bytes buffer available." );
                break;
            }

            status = epas_Read( handle, 0, offs, Buffer, len, &len );
            ErrorInfo( "ReadFile", status );
            if( status == FT_SUCCESS )
            {
                printf( "\n%04lX(%ld) bytes read.", len, len );
                HexDisplay( " DATA: ", Buffer, len );
            }
            break;

        case 'W' :
            offs = getNumber( "File Offset ......... (0..65534): " );
            p    = (unsigned char*)getString( (char*)buff, sizeof(buff),
                              "Data to write ........... (text): " );
            len  = strlen((char*)p);

            printf( "\n  %ld bytes to write.", len );
            HexDisplay( " DATA: ", p, len );

            status = epas_Write( handle, 0, offs, p, len, &len );
            ErrorInfo( "WriteFile", status );

            if( status == FT_SUCCESS )
                printf( "\n  %ld bytes written.", len );
            break;

        case 'B' :
            offs = getNumber( "File Offset ......... (0..65534): " );
            len  = getNumber( "Bytes to write ........ (2..254): " );

            if( len > sizeof(Buffer) )
            {
                printf( "\nMaximum of 256 (100h) bytes buffer available." );
                break;
            }

            for( i = 0; i < len; i++ )
            {
                unsigned char temp[40];

                sprintf( (char*)temp, "Data[%03ld] ......................: ", i );
                Buffer[i] = (unsigned char)getHexNum((char*) temp );
            }

            printf( "\n  %ld bytes to write.", len );
            HexDisplay( " DATA: ", Buffer, len );

            status = epas_Write( handle, 0, offs, Buffer, len, &len );
            ErrorInfo( "BinWriteFile", status );
            if( status == FT_SUCCESS )
                printf( "\n  %ld bytes written.", len );
            break;

        case 'I' : // Delete Dir
            df_id = getHexNum( "Dir ID ............. (1..FFFF): " );
            status = epas_DeleteDir( handle, EPAS_DIR_BY_LONG_ID, df_id, NULL );
            ErrorInfo( "DeleteDir", status );
            break;

        case 'E' : // Delete File
            ef_id = getHexNum( "File ID ............ (0..FFFF): " );
            status = epas_DeleteFile( handle, 0, ef_id );
            ErrorInfo( "DeleteFile", status );
            break;

        case 'L' :
            printf( "\nDir ID   File ID  Type Read   Write  Crypt  Size   Name"
                    "\n-------- -------- ---- ------ ------ ------ ------ ----" );
            epas_EnumDirEntries( handle, 0 );

            break;

        case 'A':
            {
                EPAS_DIRINFO di;
                unsigned char AppName[EPAS_APP_NAME_SIZE];
                unsigned char AppGuid[40];

                // Application label tag

                getString( (char*)AppName, EPAS_APP_NAME_SIZE,"Application Label .. (max 16 chars): " );
                getString( (char*)AppGuid, 40,"Application GUID ... {xxxx-xxxx...}: " );

                // We use the auto-generated dir ID for the application.

                di.ulID = 0;

                di.ulFlags      = 0;

                status = epas_CreateDir( handle,
                    EPAS_CREATE_AUTO_ID|EPAS_DIR_BY_GUID,
                    AppName[0] ? AppName : NULL,
                    AppGuid[0] ? AppGuid : NULL,
                    &di,
                    sizeof(di) );
                ErrorInfo( "CreateDir", status );
            }
			break;

        case 'D':
		{
        		EPAS_DIRINFO di;
			di.ulID         = getHexNum( "Dir ID ................. (0..FFFF): " );
                        di.ulFlags      = 0;

			status = epas_CreateDir( handle,
		        		0,
					NULL,
					NULL,
					&di,
					sizeof(di) );
			ErrorInfo( "CreateDir", status );
		}
            break;

        case 'F':
            fi.ulFlags = 0;
            fi.ulID = getHexNum( "File ID ............................. (0..FFFF): " );
            fi.ulFileSize =
                    getNumber( "Size .................................. (bytes): " );
            fi.ucFileType = (unsigned char)
                    getNumber( "Type: .................... (2=DATA 3=CTR 4=KEY): " );

            fi.ucReadAccess  = EPAS_ACCESS_ANYONE;
            fi.ucCryptAccess = EPAS_ACCESS_ANYONE;

            if( fi.ucFileType == EPAS_FILETYPE_DATA ||
                fi.ucFileType == EPAS_FILETYPE_COUNTER )
                fi.ucReadAccess = (unsigned char)
                    getNumber( "Read: ........ (0=Always 1=PIN 2=SOPIN 7=Never): " );

            fi.ucWriteAccess = (unsigned char)
                    getNumber( "Write: ....... (0=Always 1=PIN 2=SOPIN 7=Never): " );

            if( fi.ucFileType == EPAS_FILETYPE_KEY )
                fi.ucCryptAccess = (unsigned char)
                    getNumber( "Crypt: ....... (0=Always 1=PIN 2=SOPIN 7=Never): " );

            status = epas_CreateFile( handle, 0, &fi, sizeof(fi) );
            ErrorInfo( "CreateFile", status );
            break;

        case 'Y' :
            CryptMenu();
            break;

        case 'U':
            SetupMenu();
            break;

        case 'X':
            return;
    }
}

int
main( int    argc,
      char * argv[] )
{
        EPAS_STATUS status;
        unsigned long ver, Flags;
        unsigned char Buffer[80];

        printf( "Feitian ePass editor version 2.0.0\n"
                "Created: " __DATE__ "  " __TIME__ "\n" );
        ver = epas_GetDriverVersion();
        printf( "\nDriver version %ld.%ld.%ld",
                (ver >> 24) & 0xff,         // Major version
                (ver >> 16) & 0xff,         // Minor version
                (ver & 0xffff) );           // Build number

        ver = epas_GetLibVersion();
        printf( "\nLibrary version %ld.%ld.%ld",
                (ver >> 24) & 0xff,         // Major version
                (ver >> 16) & 0xff,         // Minor version
                (ver & 0xffff) );           // Build number

        status = epas_CreateContext( &handle, 0, EPAS_API_VERSION );

        ErrorInfo( "CreateHandle", status );

        for( ;; )
                switch( GetMenuItem(
                        "MAIN: Open(F)irst  LED(O)n   (P)in     (L)ist       File(M)enu\n"
                        "      Open(N)ext   LE(D)Off  (S)oPin   (G)enRandom  Cr(y)ptMenu\n"
                        "      (R)eOpen     Rese(t)   (A)ccess               Set(u)pMenu\n"
                        "      (C)lose      E(x)it" ))
                {
                case 'A':
                {
                        EPAS_ACCESSINFO accessInfo;

                        status = epas_GetProperty( handle, EPAS_PROP_ACCESSINFO, NULL, &accessInfo, sizeof(accessInfo) );
                        ErrorInfo( "GetAccessSettings", status );
                        if( status == FT_SUCCESS )
                        {
                                printf( "\n  Current PIN retry counter ..... = %d"
                                        "\n  Maximum PIN retries ........... = %d"
                                        "\n  File/Dir create access ........ = %s"
                                        "\n  File/Dir delete access ........ = %s",
                                accessInfo.ucCurPinCounter,
                                accessInfo.ucMaxPinRetries,
                                GetFileAccessText( accessInfo.ucCreateAccess ),
                                GetFileAccessText( accessInfo.ucDeleteAccess ));
                        }
                        break;
                }
                case 'M':
                        FileMenu();
                        break;
                case 'Y' :
                        CryptMenu();
                        break;
                case 'U':
                        SetupMenu();
                        break;
                case 'O':
                        status = ErrorInfo( "LED-ON ",
                                            epas_SetProperty( handle, EPAS_PROP_LED_ON, NULL, NULL, 0 ));
                        break;
                case 'D':
                        status = ErrorInfo( "LED-OFF",
                                            epas_SetProperty( handle, EPAS_PROP_LED_OFF, NULL, NULL, 0 ));
                        break;
                case 'C':
                        status = ErrorInfo( "CloseDevice", epas_CloseDevice( handle ));
                        break;

                case 'R': Flags = EPAS_OPEN_CURRENT; goto find_it;
                case 'F': Flags = EPAS_OPEN_FIRST; goto find_it;
                case 'N': Flags = EPAS_OPEN_NEXT;
find_it:
                        status = epas_OpenDevice( handle, Flags, NULL );
                        if( status != FT_SUCCESS )
                        {
                                ErrorInfo( " OpenDevice", status );
                                break;
                        }
                        {
                                EPAS_VERSIONINFO verinfo;
                                status = epas_GetProperty( handle, EPAS_PROP_VERSIONINFO, NULL, &verinfo, sizeof(verinfo) );
                                if( status == FT_SUCCESS )
                                {
                                        printf( "\n  Firmware Version ... = %d.%d"
                                                "\n  Product Code ....... = %X",
                                                verinfo.ucFwVerMajor,
                                                verinfo.ucFwVerMinor,
                                                verinfo.ucProductCode );
                                }
                        }
                        {
                                unsigned long caps;
                                status = epas_GetProperty( handle, EPAS_PROP_CAPABILITIES, NULL, &caps, sizeof(caps) );
                                if( status == FT_SUCCESS )
                                {
                                        printf( "\n  Capabilities ....... = %lx", caps );
                                }
                        }
                        {
                                unsigned long MemorySize;
                                status = epas_GetProperty( handle, EPAS_PROP_MEM_SIZE, NULL, &MemorySize, sizeof(MemorySize) );
                                if( status == FT_SUCCESS )
                                {
                                        printf( "\n  Memory Size ........ = %ld (bytes)", MemorySize );
                                }
                        }
                        {
                                EPAS_SYSINFO SysInfo;
                                status = epas_GetProperty( handle, EPAS_PROP_SYSINFO, NULL, &SysInfo, sizeof(SysInfo) );
                                if( status == FT_SUCCESS )
                                {
                                        printf( "\n  Free Space ......... = %ld (bytes)", SysInfo.ulFreeSpace );
                                        printf( "\n  File System ........ = %d", SysInfo.ucFileSysType );
                                        printf( "\n  Max Dir Levels ..... = %d", SysInfo.ucMaxDirLevels );
                                }
                        }
                        {
                                status = epas_GetProperty( handle, EPAS_PROP_SERNUM, NULL, SerialNumber, sizeof(SerialNumber) );
                                if( status == FT_SUCCESS )
                                {
                                        printf( "\n  Serial No .......... = %08lX%08lX",
                                        SerialNumber[1], SerialNumber[0] );
                                }
                        }
                        {
                                status = epas_GetProperty( handle,
                                EPAS_PROP_FRIENDLY_NAME,
                                NULL,
                                Buffer,
                                sizeof(Buffer) );
                                if( status == FT_SUCCESS )
                                {
                                        printf( "\n  Token Name ......... = '%s'", Buffer );
                                }
                        }
                        break;
                case 'G' :  // epas_GenRandom
                        status = epas_GenRandom( handle, 0, Buffer, 8 );
                        ErrorInfo( "GenRandom", status );
                        if( status == FT_SUCCESS ) HexDisplay( "GenRandom: ", Buffer, 8 );
                        break;

                case 'L' :
                        printf( "\nDir ID   File ID  Type Read   Write  Crypt  Size   Name"
                                "\n-------- -------- ---- ------ ------ ------ ------ ----" );
                        epas_EnumDirEntries( handle, 0 );
                        break;
                case 'T' :  // Reset
                        ErrorInfo( "RESET",
                                  epas_ResetSecurityState( handle, EPAS_SCOPE_MF ));
                        break;
                case 'P':
                        PutInt32( buff, getHexNum( "Enter PIN: " ));
                        status = epas_Verify( handle, EPAS_VERIFY_USER_PIN, buff, 4 );
                        ErrorInfo( "VerifyPin", status );
                        break;
                case 'S':
                        getString((char*) buff, sizeof(buff), "Enter SO-PIN: " );
                        status = epas_Verify( handle,
                                                EPAS_VERIFY_SO_PIN,
                                                buff,
                                                strlen((char*)buff) );
                        ErrorInfo( "VerifySOPIN", status );
                        break;
                case 'X':
                        epas_CloseDevice( handle );
                        epas_DeleteContext( handle );
                        return 0;
                }
                return 0;
}

